/**
 * @file
 */

(function ($) {
    'use strict';
    Drupal.behaviors.handsontablejson = {
        attach: function (context, settings) {

            var renderedTables = $(document).find('.rendered-table.format-json');

            renderedTables.each(function () {

                var textarea = $(this).parents('.field--type-table').find('input[type="hidden"]');
                if ((($(textarea).val()).length > 0) && (($(this).html()) == '')) {
                    var config = $(this).parents('.field--type-table').find('.table-config');
                    var rows = 1;
                    var columns = 2;

                    if ((config.length > 0)
                        && (($(config).val()).length > 0)
                        && (($(this).html()) == '')) {
                        config = JSON.parse(config.val());
                        rows = config.rows !== undefined ? config.rows : 1;
                        columns = config.columns !== undefined ? config.columns : 2;
                        config = config.value;
                    }
                    var data = JSON.parse($(textarea).val());
                    var header = data[0];
                    var table = new Handsontable(this, {
                        data: data,
                        cells: function (row, col, prop) {
                            var cellProperties = {};
                            if (config[row] !== undefined && config[row][col] !== undefined) {
                                cellProperties.placeholder = config[row][col];
                            }

                            return cellProperties;
                        },
                        minRows: rows,
                        minCols: columns,
                        stretchH: 'all',
                        contextMenu: {
                            items: {
                                'row_above': {},
                                'row_below': {},
                                'hsep1': "---------",
                                'col_left': {},
                                'col_right': {},
                                'hsep2': '---------',
                                'remove_row': {},
                                'remove_col': {},
                                'hsep3': '---------',
                                "copy_paste": {
                                    "name": Drupal.t('Copy & Paste'),
                                    "callback": function (name, selected, clickpoint) {
                                        for (var row = selected.start.row; row <= selected.end.row; row++) {
                                            for (var col = selected.start.col; col <= selected.end.col; col++) {
                                                var value = this.getCellMeta(row, col).placeholder;
                                                if (value) {
                                                    this.setDataAtCell(row, col, value);
                                                }
                                            }
                                        }
                                    }
                                }

                            }
                        }
                    });

                    table.updateSettings({
                        afterChange: function (e) {
                            $(textarea).val(JSON.stringify(table.getData()));
                        },
                        afterCreateRow: function (index, amount) {
                            $(textarea).val(JSON.stringify(table.getData()));
                        },
                        afterRemoveCol: function (index, amount) {
                            $(textarea).val(JSON.stringify(table.getData()));
                        },
                        afterRowMove: function (index, amount) {
                            $(textarea).val(JSON.stringify(table.getData()));
                        },
                        afterRemoveRow: function (index, amount) {
                            $(textarea).val(JSON.stringify(table.getData()));
                        },
                        afterCreateCol: function (index, amount) {
                          $(textarea).val(JSON.stringify(table.getData()));
                        }
                    });
                }

            });

        }
    };
}(jQuery));
