<?php

namespace Drupal\blizz_table_field;

use Drupal\blizz_table_field\Event\MarkdownEvents;
use Drupal\blizz_table_field\Event\TextToChangeEvent;
use League\CommonMark\CommonMarkConverter;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Class MarkdownExtension.
 *
 * @package Drupal\blizz_table_field
 */
class MarkdownExtension implements MarkdownInterface {

  /**
   * The markdown converter.
   *
   * @var \League\CommonMark\CommonMarkConverter
   */
  private $markdown;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  private $dispatcher;

  /**
   * MarkdownExtension constructor.
   *
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher
   *   The event dispatcher.
   */
  public function __construct(EventDispatcherInterface $dispatcher) {
    $this->markdown = new CommonMarkConverter();
    $this->dispatcher = $dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public function convertToHtml($commonMark) {
    $event = new TextToChangeEvent($commonMark);

    $this->dispatcher->dispatch(MarkdownEvents::PRE_TEXT_CHANGE, $event);
    $event->addText($this->markdown->convertToHtml($event->getLatestText()));
    $this->dispatcher->dispatch(MarkdownEvents::POST_TEXT_CHANGE, $event);
    return $event->getLatestText();
  }

}
