<?php

namespace Drupal\blizz_table_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'table_widget' widget.
 *
 * @FieldWidget(
 *   id = "table_widget",
 *   label = @Translation("Table widget"),
 *   field_types = {
 *     "table"
 *   }
 * )
 */
class TableWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'size' => 60,
      'placeholder' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = [];

    $elements['size'] = [
      '#type' => 'number',
      '#title' => t('Size of textfield'),
      '#default_value' => $this->getSetting('size'),
      '#required' => TRUE,
      '#min' => 1,
    ];

    $elements['placeholder'] = [
      '#type' => 'textfield',
      '#title' => t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $placeholder_setting = $this->getSetting('placeholder');
    $summary[] = t('Textfield size: !size', ['!size' => $this->getSetting('size')]);
    if (!empty($placeholder_setting)) {
      $summary[] = t('Placeholder: @placeholder', ['@placeholder' => $placeholder_setting]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = [];

    $format = $items->getSetting('format');

    $default['json'] = "[\r\n[\r\n\"Column 1\",\r\n\"Column 2\",\r\n\"Column 3\",\r\n\"Column 4\"\r\n],\r\n[\r\n\"\",\r\n\"\",\r\n\"\",\r\n\"\"\r\n],\r\n[\r\n\"\",\r\n\"\",\r\n\"\",\r\n\"\"\r\n],\r\n[\r\n\"\",\r\n\"\",\r\n\"\",\r\n\"\"\r\n],\r\n[\r\n\"\",\r\n\"\",\r\n\"\",\r\n\"\"\r\n ]\r\n]";
    $default['csv'] = " Column 1, Column 2, Column 3, Column 4 \r\n,,,\r\n,,,\r\n,,,\r\n,,,";

    $element['value'] = $element + [
      '#type' => 'hidden',
      '#description' => t('The serialized table data.'),
      '#size' => $this->getSetting('size'),
      '#placeholder' => $this->getSetting('placeholder'),
      '#weight' => -15,
    ];

    $element['value']['#attached']['library'] = ['blizz_table_field/handsontable-' . $format];
    $element['value']['#default_value'] = (isset($items[$delta]->value)) ? $items[$delta]->value : $default[$format];

    $element['pseudo'] = array(
      '#type' => 'item',
      '#title' => t('Table data'),
    );

    $element['table'] = [
      '#markup' => '<div class="rendered-table format-' . $format . '"></div>',
      '#title' => t('Table data'),
      '#weight' => 15,
    ];

    $description = \Drupal::config('blizz_table_field.settings')->get('formatting_options');

    $element['description'] = array(
      '#type' => 'details',
      '#title' => t('Formatting options'),
      '#description' => $description,
      '#weight' => 20,
    );

    return $element;
  }

}
